//@flow
import { extendObservable, observable, action, computed } from 'mobx';
import { User } from '../domain/Entities';
import * as Server from '../api/Server';


class UserStore {
    users: Array<User>;
    addUsers: (users: Array<User>) => void;
    changeUser: (user: User) => void;

    constructor() {
        extendObservable(this, {
            users: [],
            addUsers: action(users => {
                this.users.push(...users);
            }),
            changeUser: action(changed => {
                const user = this.users.find(user => user.id === changed.id);
                if (user) {
                    user.online = changed.online;
                }
            })
        })
    }


    async fetchUsers() {
        const users = (await Server.get('users'): any);
        this.addUsers(users);
    }

    subscribeToUserEvents() {
        Server.subscribeToUserChanges(changedUser => {
            this.changeUser(changedUser);
        });
    }

    unsubscribeUserEvents() {
        Server.unsubscribeUser();
    };

}

const userStore = new UserStore();
export default userStore;