//@flow
/**
 * @author Sven Koelpin
 */

import { Message } from '../domain/Entities';
import type { ChatNormalized } from '../domain/Entities';
import * as Server from '../api/Server';

export type ChatActionTypes =
    { type: typeof CHAT_OPEN_NEW, payload: ChatNormalized } |
    { type: typeof CHAT_RECEIVE_MESSAGE, payload: Message } |
    { type: typeof CHAT_MARK_AS_READ, payload: number };

export const CHAT_MARK_AS_READ = 'CHAT_MARK_AS_READ';
export const markChatAsRead = (chatId: number) => ({
    type: CHAT_MARK_AS_READ,
    payload: chatId
});

export const CHAT_OPEN_NEW = 'CHAT_OPEN_NEW';
export const openNewChat = (chat: ChatNormalized) => ({
    type: CHAT_OPEN_NEW,
    payload: chat
});

export const CHAT_RECEIVE_MESSAGE = 'CHAT_RECEIVE_MESSAGE';
export const receiveMessage = (message: Message) => ({
    type: CHAT_RECEIVE_MESSAGE,
    payload: message
});


export const subscribeToChatMessages = () => (dispatch: Function, getState: Function) => {
    Server.subscribeToChatMessages((nextMessage: Message) => {
        const chatForMessage: ChatNormalized = getState().ChatReducer.find(chat => chat.id === nextMessage.user);
        if (chatForMessage) {
            dispatch(receiveMessage(nextMessage));
        } else {
            dispatch(openNewChat({
                id: nextMessage.user,
                user: nextMessage.user,
                messages: [nextMessage],
                unread: true
            }));
        }
    });
};

export const unsubscribeChatMessages = () => {
    Server.unsubscribeChat();
};