//@flow
/**
 * @author Sven Koelpin
 */

import type { ChatNormalized, Message } from '../domain/Entities';
import type { ChatActionTypes } from './ChatActions';
import * as ChatActions from './ChatActions';


export type ChatReducerState = Array<ChatNormalized>;

const INIT_STATE: ChatReducerState = [];


export default (state: ChatReducerState = INIT_STATE, action: ChatActionTypes = ({}: any)): ChatReducerState => {
    switch (action.type) {
        case ChatActions.CHAT_OPEN_NEW:
            return onOpenChat(state, action.payload);
        case ChatActions.CHAT_RECEIVE_MESSAGE:
            return onReceiveMessage(state, action.payload);
        case ChatActions.CHAT_MARK_AS_READ:
            return onMarkChatAsRead(state, action.payload);
        default:
            return state;
    }
};

const onOpenChat = (state: ChatReducerState, chat: ChatNormalized): ChatReducerState => {
    return [...state, chat];
};

const onReceiveMessage = (state: ChatReducerState, message: Message): ChatReducerState => {
    return state.map(chat => {
        if (chat.id === message.user) {
            return Object.assign(({}: any), chat, {messages: [...chat.messages, message], unread: true});
        }
        return chat;
    });
};

const onMarkChatAsRead = (state: ChatReducerState, chatId: number): ChatReducerState => {
    return state.map(chat => {
        if (chat.id === chatId) {
            return Object.assign(({}: any), chat, {unread: false});
        }
        return chat;
    });
};


