//@flow
/**
 * @author Sven Koelpin
 */

import type { User } from '../domain/Entities';
import * as Server from '../api/Server';

export type UserActionTypes = UserFetchAction | UserChangedAction;

export const USERS_FETCH = 'USERS_FETCH';
type UserFetchAction = { type: typeof USERS_FETCH, payload: { [userId: number]: User } };
export const fetchUsers = () => async (dispatch: Function) => {
    const users = (await Server.get('users'): any);

    const userFetchAction: UserFetchAction = {
        type: USERS_FETCH,
        payload: users.reduce((result, user) => {
            result[user.id] = user;
            return result;
        }, {})
    };
    dispatch(userFetchAction);
};


export const USER_CHANGED = 'USER_CHANGED';
type UserChangedAction = { type: typeof USER_CHANGED, payload: User };
export const subscribeToUserEvents = () => (dispatch: Function, getState: Function) => {
    Server.subscribeToUserChanges(changedUser => {
        dispatch({
            type: USER_CHANGED,
            payload: changedUser
        });
    })
};

export const unsubscribeUserEvents = () => {
    Server.unsubscribeUser();
};
